/**************************************************************************/
/*                                 Flow4J                                 */
/*              A visual process flow configuration framework             */
/*                          (c) Alex Greif 2002                           */
/*                                                                        */
/*  This program is free software; you can redistribute it and/or modify  */
/*  it under the terms of the GNU General Public License as published by  */
/*  the Free Software Foundation; either version 2 of the License, or     */
/*  (at your option) any later version.                                   */
/*                                                                        */
/**************************************************************************/

package flow4j.servlet;

import java.io.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;

import flow4j.flow.*;



/**	The FlowServlet calls the specified flow that is given in the url
 path info. Before execution all request parameters are transformed to the new
 dictionary and afterwords the parameters are transformed from the dictionary
 back to the request.
 The url to call the servlet is of the form:
 <pre>
 http://server:port/context/FlowServlet/Flow-Start?arg1=1&...
 </pre>
 Context is the context of the webapplication
 FlowServlet can be any mapping to this or an overwritten servlet
 Flow-Entry is the name of the specific flow and its entry point, separated by a hyphen

 */
public class FlowServlet extends HttpServlet {



	/**	Handle the get-request
	 */
	public void doGet(HttpServletRequest req, HttpServletResponse res)
		throws ServletException, IOException
	{
		doGetOrPost(req, res);
	}


	/**	Handle the post-request
	 */
	public void doPost(HttpServletRequest req, HttpServletResponse res)
		throws ServletException, IOException
	{
		doGetOrPost(req,res);
	}


	/**	Calls the specific flow. Transfers all request parameters to the dictionary
	 before call and transfers them back after flow execution. The request
	 will be forwarded to the page specified in the flow.
	 */
	protected void doGetOrPost(HttpServletRequest req, HttpServletResponse res)
		throws ServletException, IOException
	{
		//	fill dictionary
		FlowDictionary dictionary = new FlowDictionary();
		transferParameters(req, res, dictionary);

		//	call flow
		FlowMgr.start(getFlowStartDesc(req), dictionary);

		//	determine dialog
		String templateName = dictionary.getTemplateName();
		if (templateName == null)
			throw new FlowServletException("No template specified");

		//	transfer dictionary contents back to the request for possible usage
		//	in jsp
		transferParameters(dictionary, req);

		//	forward request
		StringBuffer forwardPath = new StringBuffer();
		if (! templateName.startsWith("/"))
			forwardPath.append("/");

		forwardPath.append(templateName);

		forward(req, res, forwardPath.toString());
	}




	/**	Forwards the request
	 */
	private void forward(HttpServletRequest req,
						 HttpServletResponse res,
						 String forwardPath)
		throws IOException, ServletException {

		// Create a RequestDispatcher the corresponding resource
		RequestDispatcher rd =
			getServletContext().getRequestDispatcher(forwardPath);
		if (rd == null)
			throw new FlowServletException("Cannot forward to: " + forwardPath);

		// Forward control to the specified resource
		rd.forward(req, res);
	}



	/**	Transfers request parameters: the request itself, the response and the servlet
	 to the dictionary
	 */
	private void transferParameters(HttpServletRequest req,
									HttpServletResponse res,
									FlowDictionary dictionary)
	{
		for (Enumeration enum = req.getParameterNames(); enum.hasMoreElements();) {
			String name = (String)enum.nextElement();
			String[] value = req.getParameterValues(name);

			if (value.length == 1)
				dictionary.put(name, value[0]);
			else
				dictionary.put(name, value);
		}

		dictionary.put("request", req);
		dictionary.put("response", res);
		dictionary.put("servlet", this);
	}


	/**	transfers dictionary parameters to the request.
	 */
	private void transferParameters(FlowDictionary dictionary, HttpServletRequest req) {
		for (Iterator iter = dictionary.entrySet().iterator(); iter.hasNext();) {
			Map.Entry entry = (Map.Entry)iter.next();
			req.setAttribute(entry.getKey().toString(), entry.getValue());
		}
	}


	/**	Extracts the method name out of the url.
	 */
	protected String getFlowStartDesc(HttpServletRequest req)
		throws FlowServletException
	{
		if (req.getPathInfo() == null)
			throw new FlowServletException("no flow and entry specified");


		StringTokenizer tokens = new StringTokenizer(req.getPathInfo(),"/");
		if (tokens.hasMoreTokens())
			return tokens.nextToken();

		throw new FlowServletException("no flow and entry specified");
	}












	/**
	 */
	class FlowServletException extends ServletException {

		FlowServletException(String reason) {
			super(reason);
		}

	}





}


